#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <string.h>

struct point{
	double x;
	double y;
};

double ask_point_depth(char *str, int i);
double ask_positive_double(char *str);
int ask_positive_int(char *str);
int left_deepest(struct point *set, struct point deepest, int size);
int right_deepest(struct point *set, struct point deepest, int size);
char ask_for_repeat(void);
int left_turn(struct point *pts, struct point *res, double len, int size);
int right_turn(struct point *pts, struct point *res, double len, int size);
void show_results(struct point *l_res, struct point *r_res, double len, int l_size, int r_size);

int
main()
{
	double interval, max_depth, range, len;
	int n, i, deepest, left_size, right_size;
	int *res;

	struct point *pts, *left_res, *right_res;

	max_depth = 0;

	/*primary input block*/
	n = ask_positive_int("Enter the number of points: ");
	interval = ask_positive_double("Enter the interval between points: ");
	len = ask_positive_double("Enter the length of a piece of pipe: ");
	range = ask_positive_double("Enter the range to bottom: ");

	/*associating pointers with memory areas*/
	res = malloc(sizeof(*res));
	if (res == NULL) {
		printf("You need more the memory\n");
		exit(5);
	}

	pts = malloc((n+2) * sizeof(*pts));
	if (pts == NULL) {
		printf("Memory allocating for points array failed\n");
		exit(6);
	}

	left_res = malloc((n+2) * sizeof(*left_res));
	if (left_res == NULL) {
		printf("Memory allocating for l_results array failed\n");
		exit(6);
	}

	right_res = malloc((n+2) * sizeof(*right_res));
	if (right_res == NULL) {
		printf("Memory allocating for r_results array failed\n");
		exit(6);
	}

	/*secondary input block*/
	for (i = 0; i < n; i++) {
		pts[i+1].y = ask_point_depth("Enter the depth in point ", i+1);

		/*get maximal depth value*/
		if (pts[i+1].y >= max_depth) {
			deepest = i+1;
			max_depth = pts[i+1].y;
		}

		/*increase depth value on range to bottom*/
		pts[i+1].y += range;
		pts[i+1].x = i*interval;
	}

	pts[0].y = 0;
	pts[n+1].y = 0; 
	pts[0].x = pts[1].x;
	pts[n+1].x = pts[n].x;
	left_res[0].x = pts[deepest].x;
	left_res[0].y = pts[deepest].y;
	right_res[0].x = pts[deepest].x;
	right_res[0].y = pts[deepest].y;

	/*these 2 functions are calculating
	angles and depts values in each point*/
	left_size = left_turn(pts, left_res, len, n+2);
	right_size = right_turn(pts, right_res, len, n+2);

	/*output block*/
	show_results(left_res, right_res, len, left_size, right_size);

	/*releazing resources*/
	free(res);
	free(pts);

	return 0;
}

/*function getting a non-negative double value
from user printing message *str. Function prints
'i' value if 'i' argument isn't equal 0*/
double
ask_point_depth(char *str, int i)
{
	double value;
	int res;
	char *msg, def;

	def = '\0';
	msg = &def;
	value = -1;

	if (str != NULL)
		if (strlen(str) > 0)
			msg = str;

	while (value < 0) {
		printf(msg);

		if (i != 0) {
			printf("%i: ", i);
		}

		res = scanf("%lf", &value);
		if (res < 1) {
			printf("You entered some trash\n");

			if (ask_for_repeat() == 'y')
				continue;
			exit(1);
		}
		if (value < 0) {
			printf("There can't be a negative value\n");

			if (ask_for_repeat() == 'y')
				continue;
			exit(2);
		}
	}

	return value;
}

/*funcion getting a positive double number
from user printing message *str*/
double
ask_positive_double(char *str)
{
	double value;

	value = -1;

	while (value <= 0) {
		value = ask_point_depth(str, 0);
		if (value == 0) {
			printf("A number must be positive\n");

			if (ask_for_repeat() == 'y')
				continue;
			exit(3);
		}
	}

	return value;
}

/*function getting positive integer number from
user printing message *str*/
int
ask_positive_int(char *str)
{
	double value, difference;

	value = -1;

	while (value <= 0) {
		value  = ask_positive_double(str);
		difference = value - (int)value;
		
		if (difference != 0) {
			printf("A number must be integer\n");
			
			if (ask_for_repeat() == 'y')
				continue;
			exit(4);
		}
	}

	return (int)value;
}

/*function returns the index of the deepest point
before the deepest point*/
int
left_deepest(struct point *set, struct point deepest, int size)
{
	int i, max;
	double max_val;

	max = 0;
	max_val = -1;

	for (i = size-1; i >= 0; i--) {
		if (set[i].x < deepest.x)
			if (set[i].y <= deepest.y)
				if (set[i].y >= max_val) {
					max = i;
					max_val = set[i].y;
				}
	}

	return max;
}

/*function returns the index of the deepest point
after the deepest point*/
int
right_deepest(struct point *set, struct point deepest, int size)
{
	int i, max;
	double max_val;

	max = size - 1;
	max_val = 0;

	for (i = 0; i < size; i++) {
		if (set[i].x > deepest.x)
			if (set[i].y <= deepest.y)
				if (set[i].y >= max_val) {
					max = i;
					max_val = set[i].y;
				}
	}

	return max;
}

/*this function asks user to one more attempt*/
char
ask_for_repeat(void)
{
	int res;
	char c;

	while (getchar() != '\n') {}
	printf("Enter 'y' to try again: ");

	res = scanf("%c", &c);
	if (res > 0) {
		res = tolower(c);
		return c;
	}
	return '\0';
}

/*function filling with values a part of depts
and angles arrays before the deepest point*/
int
left_turn(struct point *pts, struct point *res, double len, int size)
{
	double hyp, h, w, splitted;
	int next, i;

	next = -1;

	for (i = 1; next != 0; i++) {
		next = left_deepest(pts, res[i-1], size);
		h = res[i-1].y - pts[next].y;
		w = res[i-1].x - pts[next].x;

		if (180*atan2(h, w)/M_PI <= 7) {
			hyp = hypot(w, h);
			splitted = len * (int)(hyp / len);

			if (hyp >= len) {
				if (hyp == splitted) {
					res[i].x = res[i-1].x - w;
					res[i].y = res[i-1].y - h;
				} else {
					splitted += len;
					w = splitted * cos(asin(h/splitted));
					res[i].x = res[i-1].x - w;
					res[i].y = res[i-1].y - h;
				}
			} else {
				w = len * cos(asin(h/len));
				res[i].x = res[i-1].x - w;
				res[i].y = res[i-1].y - h;
			}
		} else {
			w = h / tan(M_PI*7/180);
			hyp = hypot(w, h);
			splitted = len * (int)(hyp / len);

			if (hyp >= len) {
				if (hyp == splitted) {
					res[i].x = res[i-1].x - w;
					res[i].y = res[i-1].y - h;
				} else {
					splitted += len;
					w = splitted * cos(asin(h/splitted));
					res[i].x = res[i-1].x - w;
					res[i].y = res[i-1].y - h;
				}
			} else {
				w = len * cos(asin(h/len));
				res[i].x = res[i-1].x - w;
				res[i].y = res[i-1].y - h;
			}
		}
	}

	return i-1;
}

/*function filling with values a part of depts and angles
arrays after the deepest point including this point*/
int
right_turn(struct point *pts, struct point *res, double len, int size)
{
	double hyp, h, w, splitted;
	int next, i;
	
	next = -1;

	for (i = 1; next != (size-1); i++) {
		next = right_deepest(pts, res[i-1], size);
		h = res[i-1].y - pts[next].y;
		w = pts[next].x - res[i-1].x;

		if (180*atan2(h, w)/M_PI <= 7) {
			hyp = hypot(w, h);
			splitted = len * (int)(hyp / len);

			if (hyp >= len) {
				if (hyp == splitted) {
					res[i].x = res[i-1].x + w;
					res[i].y = res[i-1].y - h;
				} else {
					splitted += len;
					w = splitted * cos(asin(h/splitted));
					res[i].x = res[i-1].x + w;
					res[i].y = res[i-1].y - h;
				}
			} else {
				w = len * cos(asin(h/len));
				res[i].x = res[i-1].x + w;
				res[i].y = res[i-1].y - h;
			}
		} else {
			w = h / tan(M_PI*7/180);
			hyp = hypot(w, h);
			splitted = len * (int)(hyp / len);

			if (hyp >= len) {
				if (hyp == splitted) {
					res[i].x = res[i-1].x + w;
					res[i].y = res[i-1].y - h;
				} else {
					splitted += len;
					w = splitted * cos(asin(h/splitted));
					res[i].x = res[i-1].x + w;
					res[i].y = res[i-1].y - h;
				}
			} else {
				w = len * cos(asin(h/len));
				res[i].x = res[i-1].x + w;
				res[i].y = res[i-1].y - h;
			}
		}
	}

	return i-1;
}

/*function printing results of calculations*/
void
show_results(struct point *l_res, struct point *r_res, double len, int l_size, int r_size)
{
	double h, w, hyp;
	int i, j;

	j = 1;

	printf("\nMachine must enter the earth in %.4lf to river.\n", fabs(l_res[l_size].x));

	for (i = l_size; i > 0; i--) {
		h = l_res[i-1].y - l_res[i].y;
		w = l_res[i-1].x - l_res[i].x;
		hyp = hypot(w, h);

		printf("\nVector %i:\n\t%i pieces of pipe\n", j++, (int)(hyp/len));
		if (atan2(h, w) != 0) {
			printf("\t%.4lf degrees to the horizont", -180*atan2(h, w)/M_PI);
		} else {
			printf("\t%.4lf degrees to the horizont", 180*atan2(h, w)/M_PI);
		}
	}	
	for (i = r_size; i > 0; i--) {
		h = r_res[i-1].y - r_res[i].y;
		w = r_res[i].x - r_res[i-1].x;
		hyp = hypot(w, h);

		printf("\nVector %i:\n\t%i pieces of pipe\n", j++, (int)(hyp/len));
		printf("\t%.4lf degrees to the horizont", 180*atan2(h, w)/M_PI);
	}	

	printf("\n\nMachine will leave the earth in %.4lf to river.\n", fabs(r_res[r_size-1].x - r_res[r_size].x));

	printf("\nEnter 'E' or 'e' for exit\n\n");

	while (tolower(getchar())!= 'e') {}
}
